from typing import List, Union, Literal, Optional, Dict, Iterable

from openai.types.chat import ChatCompletionMessageParam, ChatCompletionToolParam, ChatCompletionToolChoiceOptionParam
from pydantic import BaseModel

from aily_core import action

ACTION_API_NAME = 'action:brn:cn:spring:all:all:connector_action_runtime:/spring_sdk_llm'


class Function(BaseModel):
    arguments: Optional[str] = None

    name: Optional[str] = None


class MessageToolCall(BaseModel):
    id: str

    function: Function

    type: Literal["function"]


class Usage(BaseModel):
    input_tokens: int

    output_tokens: int


class Message(BaseModel):
    content: Optional[str] = None
    role: Literal["assistant", "user", "system"]

    tool_calls: Optional[List[MessageToolCall]] = None

    usage: Optional[Usage] = None


def convert_tool_type(messages):
    for message in messages:
        if 'tool_calls' in message:
            for tool_call in message['tool_calls']:
                tool_call['tool_type'] = tool_call['type']
    return messages


def chat_completion(
        messages: Iterable[ChatCompletionMessageParam],
        model: Union[
            str,
            Literal[
                "BYOM-lite",
                "BYOM-plus",
                "BYOM-pro",
                "BYOM-max",
                "BYOM-ultra",
                "BYOM-4o",
                "3.5-Turbo",
                "3.5-Turbo-16K",
                "4-8K",
                "4-32k",
                "4-Turbo",
                "4o",
            ],
        ],
        max_tokens: Optional[int] = None,
        temperature: Optional[float] = None,
        # stream: bool = False,  # 当前并不支持
        tools: Iterable[ChatCompletionToolParam] = None,
        tool_choice: ChatCompletionToolChoiceOptionParam = "none",
        timeout: Optional[float] = None,
) -> Message:
    """
    Generates a response message based on the input messages and parameters.

    Args:
        messages: The list of input messages.
        model: The LLM model to use for generation.
        max_tokens: The maximum number of tokens to generate.
        temperature: The temperature value for generation.
        # stream: Whether to stream the response. Currently not supported.
        tools: The tools available for the model to use.
        tool_choice: The choice of tool usage. Can be "none", "auto", or "required".
        timeout: The timeout value for the API request.

    Returns:
        The generated response message.
    """
    action_data = {
        "llm_id": model,
        "chat_completion_parameters": {
            "chat_completion_messages": convert_tool_type(messages),
            "max_tokens": max_tokens,
            "temperature": temperature,
        },
    }
    if tools is not None:
        action_data['chat_completion_parameters'].update({
            "tools": tools,
            "tool_choice": tool_choice,
        })

    res = action.call_action(
        action_api_name=ACTION_API_NAME,
        action_data=action_data,
    )
    if res["choices"]:
        choice = res["choices"][0]
        message = choice["message"]
        content = message.get("content")
        role = message["role"]
        tool_calls = [
            MessageToolCall(
                id=tool_call["id"],
                function=Function(
                    name=tool_call["function"]["name"],
                    arguments=tool_call["function"]["arguments"],
                ),
                type="function",
            )
            for tool_call in message.get("tool_calls", [])
        ]
        return Message(
            content=content,
            role=role,
            tool_calls=tool_calls if tool_calls else None,
            usage=Usage(
                input_tokens=res.get('usage', {}).get('prompt_tokens'),
                output_tokens=res.get('usage', {}).get('completion_tokens'),
            )
        )
    else:
        return Message()



import json
from typing import List, Dict, Any
from aily_core import action

OQL_API_NAME = 'action:brn:cn:spring:all:all:connector_action_runtime:/spring_sdk_oql_exec'
SQL_API_NAME = 'action:brn:cn:spring:all:all:connector_action_runtime:/spring_sdk_sql_exec'


def execute_oql(oql: str, args: list = None, named_args: dict = None) -> List[Dict[str, Any]]:
    """
    执行OQL查询,用于查询数据表的内容。

    Args:
        oql (str): 要执行的OQL查询语句。
        args:
        named_args:
    Returns:
        List[Dict[str, Any]]: 查询结果,每个字典代表一行数据。
            - 字典的键为列名(str)
            - 字典的值为对应的数据(Any)

    """
    # 执行OQL查询并获取结果
    response = action.call_action(OQL_API_NAME, {
        "query": oql,
        "args": args or [],
        "namedArgs": named_args or {}
    })

    rows = []
    for row in json.loads(response['rows']):
        rows.append(row)
    return rows


def execute_sql(sql: str) -> List[Dict[str, Any]]:
    """
    执行SQL查询,用于查询分析表的内容。

    Args:
        sql (str): 要执行的SQL查询语句。

    Returns:
        List[Dict[str, Any]]: 查询结果,每个字典代表一行数据。
            - 字典的键为列名(str)
            - 字典的值为对应的数据(Any)
    """
    # 执行SQL查询并获取结果
    response = action.call_action(SQL_API_NAME, {"query": sql})

    rows = []
    for row in response['data']:
        row_data = json.loads(row)
        rows.append(row_data)
    return rows



def get_tenant_access_token(tenant_id: str, app_id: str, app_secret: str) -> str:
    """
    获取应用访问令牌。

    Args:

    Returns:
        str: 应用访问令牌,用于验证应用对资源的访问权限。
    """
    pass


def get_user_access_token() -> str:
    """
    获取用户访问令牌。

    Args:

    Returns:
        str: 用户访问令牌,用于验证用户对资源的访问权限。
    """
    pass



from dataclasses import dataclass
from aily_core import action
from typing import List, Optional

KNOWLEDGE_API_NAME = 'action:brn:cn:spring:all:all:connector_action_runtime:/spring_sdk_wiki_recall'


@dataclass
class Chunk:
    content: str
    score: float
    chunk_id: str
    raw_data: dict
    title: Optional[str] = None
    link: Optional[str] = None


def retrieve(
        knowledge_id: str,
        query_text: str,
        top_k: int = 5,
        threshold: float = 0.5
) -> List[Chunk]:
    """
    在指定的知识库中查询与给定文本相似的条目。

    Args:
        knowledge_id (str): 要查询的知识库ID。
        query_text (str): 查询的文本内容。
        top_k (int, optional): 返回的最相似条目的数量。默认为5。
        threshold (float, optional): 相似度阈值,只返回相似度大于等于该阈值的条目。默认为0.5。

    Returns:
        List[Chunk]: 查询结果列表,每个元素是一个Chunk对象,包含以下字段:
            - content (str): 条目的内容。
            - title (str): 条目的标题。
            - score (float): 条目与查询文本的相似度得分。
            - link (str): 条目的链接。
            - chunk_id (str): 条目的ID。
    """
    if not knowledge_id:
        raise ValueError("知识库ID不能为空")

    if not query_text:
        raise ValueError("查询文本不能为空")

    if top_k <= 0:
        raise ValueError("top_k必须大于0")

    if threshold < 0 or threshold > 1:
        raise ValueError("threshold必须在[0, 1]范围内")

    response = action.call_action(KNOWLEDGE_API_NAME, {
        "apiName": knowledge_id,
        "key": query_text,
        "scoreThreshold": threshold,
        "topk": top_k,
    })
    results = []
    for chunk in response:
        source_value = chunk.get('source_value', {})
        results.append(
            Chunk(
                content=source_value.get('content'),
                title=source_value.get('doc', {}).get('title'),
                score=chunk.get('recall_score'),
                link=source_value.get('doc', {}).get('link'),
                chunk_id=chunk.get('id'),
                raw_data=chunk
            )
        )
    return results




import json
import uuid
from aily_core import action

SEND_MESSAGE_API_NAME = 'action:brn:cn:spring:all:all:connector_action_runtime:/spring_sdk_send_message'


def send_message(content: str) -> int:
    """
    发送一条消息。

    Args:
        content (str): 要发送的消息内容。

    Returns:
        int: 发送消息的结果代码。
            - 0: 发送成功
            - 1: 发送失败
            - 2: 消息内容为空
            - 3: 消息内容过长(超过1000个字符)
    """
    idempotent_id = str(uuid.uuid4())

    content = {"content": {"widgets": [{"type": "Markdown", "props": {"content": content, "resources": []}}]}}
    r = action.call_action(SEND_MESSAGE_API_NAME, {
        'idempotentID': idempotent_id,
        'message': {
            'content': json.dumps(content),
            'messageStatus': 'FINISHED',
            'steps': []
        }
    })
    return r.get('messageID')


def update_message(message_id: int, content: str) -> bool:
    """
    更新指定ID的消息内容。

    Args:
        message_id (int): 要更新的消息ID。
        content (str): 新的消息内容。

    Returns:
        bool: 更新消息的结果。
            - True: 更新成功
            - False: 更新失败
    """
    # 实际的消息更新逻辑...
    success = True  # 假设更新成功
    return success



from typing import Dict, Any


def invoke(skill_id: str, params: Dict[str, Any] = None) -> Dict[str, Any]:
    """
    调用指定技能的接口。

    Args:
        skill_id (str): 技能的唯一标识符。
        params (Dict[str, Any], optional): 调用技能接口所需的参数。
            不同的技能可能需要不同的参数,具体参数的键和值由相应的技能文档定义。

    Returns:
        Dict[str, Any]: 技能接口的返回值。
            不同的技能可能返回不同的结果,具体结果的结构和内容由相应的技能文档定义。
    """
    pass